import wx as wx
from wx.lib.mixins.listctrl import  TextEditMixin
import xml.etree.cElementTree as ET


class CustomApp(wx.App):
    def __init__(self):
        wx.App.__init__(self)
        self.steps=None
    def SetInput(self, Steps,Values):
        self.settings = dict(self.settings.items() +Values.items())
        self.steps  = Steps
    def SetAdvanced(self,array):
        self.settings=array
 
    
class AdvancedFrame(wx.Frame):
      def __init__(self,parent):
           wx.Frame.__init__(self, parent)
           vbox=wx.BoxSizer(wx.VERTICAL)
           panel = wx.Panel(self)
           self.boxes=[ CustomBox(panel,default="2.0", label="Length (mm)"),
                        CustomBox(panel,default="0.0", label="Base"),
                        CustomBox(panel,default="1.0e-6", label="Increment of strain"),
                        CustomBox(panel,default="False", label="Debug",choices=["True","False"]),
                        CustomBox(panel,default="250", label="Faces"),
                        CustomBox(panel,default="30", label="Resolution"),
                        CustomBox(panel,default="1", label="Quadrature",choices=["1","2"]),
                        CustomBox(panel,default="Static", label="Update of Mesh",choices=["Increment","Static"]),
                        CustomBox(panel,default="Closed", label="End Condition",choices=["Open","Closed"])]
                        
                        
                    
           btn=wx.Button(panel,100,"Ok",size=(90,30))
           self.Bind(wx.EVT_BUTTON,self.OnQuit,id=100)
           dfl=wx.Button(panel,99,"Reset",size=(90,30))
           self.Bind(wx.EVT_BUTTON,self.OnDefault,id=99)
           vbox.AddSpacer((-1,20))
           
           for i in self.boxes:
               vbox.Add(i,1,wx.ALL|wx.EXPAND|wx.CENTER,5)
           
           hbox=wx.BoxSizer(wx.HORIZONTAL)          
           hbox.Add(dfl,0,wx.ALIGN_RIGHT|wx.ALL,10)
           hbox.Add(btn,0,wx.ALIGN_RIGHT|wx.ALL,10)

           
           vbox.Add(hbox,0,wx.ALIGN_RIGHT|wx.ALL,10)
           vbox.Fit(self) 
           panel.SetSizer(vbox)
          
           self.Center()
           self.parent=parent
      def OnQuit(self,event):
          if self.SetValues()==True:
              self.parent.adv_frame=None
              self.Close()          
      def OnDefault(self,event):        
          [i.SetDefault() for i in self.boxes]
      
      def GetValues(self):
          return { i.GetLabel().split()[0]:i.GetValue() for i in self.boxes}

      def SetValues(self):
            try:
                wx.GetApp().SetAdvanced(self.GetValues())   
                return True
            except:
                wx.MessageBox("Please insert correct input values", "Incorrect Values",wx.OK)
                return False
      def SetDefaults(self):
         wx.GetApp().settings=self.GetValues()
         self.Close()
      

          
          
class CustomBox(wx.BoxSizer):
    def __init__(self, panel, default="0",choices=[],*args ,**kwargs):
        super(CustomBox,self).__init__(wx.HORIZONTAL)
        if not choices :
            self.value=wx.TextCtrl(panel,value=default)
        else: 
            self.value=wx.ComboBox(panel,value=default,choices=choices)
        self.default=default
        self.Rform()
        
        self.AddSpacer((50,30)) 
        self.Add(wx.StaticText(panel,*args,**kwargs) , 1, wx.ALL, 5)
        self.AddSpacer((50,30)) 
        self.Add(self.value, 1, wx.ALL|wx.EXPAND, 5)
        self.label=kwargs.get("label")
    def Rform(self):
        try:
            float(self.default)        
            self.flag=True
        except:
            self.flag=False            
    def SetValue(self,value):
        self.value.ChangeValue(value)
    def SetDefault(self):
        try:
            self.value.ChangeValue(self.default)
        except:
            self.value.SetValue(self.default)
    def GetValue(self):
        if self.flag==True:
                return float(self.value.GetValue())
        else :
                return str(self.value.GetValue())  
        
    def GetLabel(self):
        return self.label
        

class EditableListCtrl(wx.ListCtrl, TextEditMixin):
    def __init__(self, parent):
        wx.ListCtrl.__init__(self, parent, -1, style=wx.LC_REPORT,size=(500,-1))
        TextEditMixin.__init__(self)

        
class User_interface(wx.Frame):
    def __init__(self, *args, **kwargs):
        super(User_interface, self).__init__(*args, **kwargs) 
        self.config=wx.Config("APS")
        self.dirname=self.config.Read("Path") 
        self.output_folder=str(self.config.Read("Output_folder"))
        
        self.InitUI()
        
    def InitUI(self):    

        Main_pnl = wx.Panel(self, -1)
        AdvancedFrame(self).SetDefaults()
        
##############################################################################        
#           PANEL 1
##############################################################################
        pnl1=wx.Panel(Main_pnl,-1)
        vbox1=wx.BoxSizer(wx.VERTICAL)
        self.boxes=[CustomBox(pnl1, default="Insert material name", label='Material'),
                    CustomBox(pnl1, default="200.0" ,label='Young Modulus   (GPa)'),  
                    CustomBox(pnl1, default="1.0" ,label='Work-Hardening Modulus (GPa)'), 
                    CustomBox(pnl1, default="25.0",label='Outer Diameter (mm)'), 
                    CustomBox(pnl1, default="2.5",label='Wall Thickness (mm)'),
                    CustomBox(pnl1, default="0.3", label='Poisson ratio '),
                    CustomBox(pnl1, default="600",label='SMYS (MPa)')]

        vbox1.AddSpacer((-1,20))
        [vbox1.Add(i,1,wx.ALL|wx.EXPAND|wx.CENTER,5) for i in self.boxes]
        adv=wx.Button(pnl1,27,'Advanced',size=(90,30))
        vbox1.Add(adv,0,flag=wx.ALIGN_RIGHT, border=10)   
        pnl1.SetSizer(vbox1)
############################################################################# 
# Panel 2
#############################################################################
        
        pnl2= wx.Panel(Main_pnl,-1)
        hbox2=wx.BoxSizer(wx.HORIZONTAL)
        
        vbox2=wx.BoxSizer(wx.VERTICAL)
         
        text=wx.StaticText(pnl2,label="    Select output folder :")
        brw=wx.Button(pnl2,19,'Browser',size=(90,30))
        
        self.brw_tc=wx.TextCtrl(pnl2,size=(450,-1),value=self.output_folder)   

    
        
        hbox2.Add(self.brw_tc,0.1, wx.LEFT|wx.EXPAND,  border=10)
        hbox2.Add((20, -1),1,wx.EXPAND) 
        hbox2.Add(brw,0, wx.ALIGN_RIGHT|wx.EXPAND ,border=10)
        vbox2.Add(text,1,wx.LEFT|wx.ALL|wx.EXPAND)
        vbox2.Add(hbox2) 
        vbox2.Add((-1,20))
        
        #vbox2.Add((720,-1)) # --> to avoid horizontal scroll
        pnl2.SetSizer(vbox2)
##############################################################################
# PANEL 3        
##############################################################################      
        hbox = wx.BoxSizer(wx.HORIZONTAL)    
        pnl3=wx.Panel(Main_pnl,-1)
        self.list = EditableListCtrl(pnl3)
        self.list.InsertColumn(0, 'Nr.', width=30)
        self.list.InsertColumn(1, 'Curvature Y (m'+u"\u207B"+u"\u00B9"+')' , width=130)
        self.list.InsertColumn(2, 'Curvature X (m'+u"\u207B"+u"\u00B9"+')', width=130)
        self.list.InsertColumn(3, 'Pressure (MPA)', width=120)
        self.list.InsertColumn(4, 'Tension (kN)', width= 90)

        hbox.Add(self.list, 1,flag=wx.EXPAND|wx.ALL, border=50)
        
        btnPanel = wx.Panel(pnl3, -1)
        new = wx.Button(btnPanel, 20, 'New', size=(90, 30))
        dlt = wx.Button(btnPanel, 21, 'Delete', size=(90, 30))
        clr = wx.Button(btnPanel, 22, 'Clear', size=(90, 30))

             

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add((-1, 130))
        vbox.Add(new,0,wx.TOP,5)
        vbox.Add(dlt, 0, wx.TOP, 5)
        vbox.Add(clr, 0, wx.TOP, 5)

        vbox.Add((-1,150))


        btnPanel.SetSizer(vbox)
        
        hbox.Add(btnPanel, 0, flag=wx.ALIGN_RIGHT|wx.RIGHT|wx.EXPAND ,border= 20)

        pnl3.SetSizer(hbox)
        
#############################################################################
# PANEL 4
#############################################################################        
        
        pnl4= wx.Panel(Main_pnl,-1)
        hbox4=wx.BoxSizer(wx.HORIZONTAL)
        save=wx.Button(pnl4,23,'Save',size=(90,30))
        load=wx.Button(pnl4,24,'Load',size=(90,30))
        run = wx.Button(pnl4, 25, 'Run', size=(90, 30))
        close = wx.Button(pnl4, 26, 'Close', size=(90, 30))

      
        hbox4.Add(close,0,flag=wx.BOTTOM|wx.ALL,border=10)
    
        hbox4.Add(save, 0 ,flag=wx.BOTTOM|wx.ALL, border=10)
        hbox4.Add(load, 0 ,flag=wx.BOTTOM|wx.ALL, border=10)
        hbox4.Add(run,0,flag=wx.BOTTOM|wx.ALL, border=10)

        pnl4.SetSizer(hbox4)
        

#############################################################################
#   APPERANCE      
#############################################################################
        main_vbox=wx.BoxSizer(wx.VERTICAL)        
        
        main_vbox.Add(pnl1,flag=wx.ALL|wx.EXPAND ,border=30)
        main_vbox.Add(pnl2,flag=wx.ALIGN_RIGHT|wx.RIGHT |wx.EXPAND|wx.ALL,border=10)
        main_vbox.Add(pnl3,flag= wx.EXPAND,border=30)
        main_vbox.Add(pnl4,flag=wx.ALIGN_RIGHT|wx.BOTTOM,border=30) 
        
        main_vbox.Fit(self) 
        Main_pnl.SetSizer(main_vbox)
        
        self.Bind(wx.EVT_BUTTON,self.OnBrowser,id=19)  
        
        self.Bind(wx.EVT_BUTTON, self.NewItem, id=20)
        self.Bind(wx.EVT_BUTTON, self.OnDelete, id=21)
        self.Bind(wx.EVT_BUTTON, self.OnClear, id=22)
        self.Bind(wx.EVT_BUTTON,self.OnSave,id=23)
        self.Bind(wx.EVT_BUTTON,self.OnLoad,id=24)
        
        self.Bind(wx.EVT_BUTTON,self.OnRun,id=25)
        self.Bind(wx.EVT_BUTTON, self.OnClose,id=26)
        self.Bind(wx.EVT_BUTTON,self.OnAdvanced,id=27)
        


        self.adv_frame=None
        self.count=0
        self.SetTitle('Accumulated Plastic Strain User Interface')
        self.Centre()
        self.Show(True)
###############################################################################
# EVENT FUNCTIONS   
###############################################################################    
            
        
    def OnAdvanced(self,event):
        self.adv_frame=AdvancedFrame(self)
        self.adv_frame.Show()
    def NewItem(self, event):
        self.count+=1
        self.list.Append([int(self.count),0.0,0.0,0.0,0.0])

    def OnDelete(self, event):
        if self.count>0:
            self.count-=1
            self.list.DeleteItem(self.count)    
    def OnClear(self, event):
        self.count=0
        self.list.DeleteAllItems()

    def OnClose(self,e):
        if self.adv_frame!=None:
            self.adv_frame.Close()
        self.Destroy()
        
    def OnRun(self,e):
        if self.SetData():
            if self.adv_frame!=None:
                self.adv_frame.Close()
            self.Destroy()
        
    def OnBrowser(self,event):
        dialog = wx.DirDialog(None, "Choose a directory:",style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dialog.ShowModal() == wx.ID_OK:
            self.output_folder=str(dialog.GetPath())
            self.config.DeleteGroup("Output_folder")
            self.config.Write("Output_folder",dialog.GetPath())
            self.brw_tc.ChangeValue(dialog.GetPath())

        dialog.Destroy() 

        
    def OnLoad(self,event):
         self.LoadXML()
           
        
    def OnSave(self,event):
        self.WriteXML()        
        
##############################################################################
#     Other Functions
##############################################################################     
    def SetSteps(self,Steps):
        self.count=0
        self.list.DeleteAllItems()
        for i in Steps:
            self.list.Append([(i[j],int(i[j]))[int(j==0)] for j in range(len(i))])
        self.count=len(Steps)    
        
    def SetBoxes(self,array):
        [self.boxes[i].SetValue(array[i]) for i in range(len(self.boxes))] 
        
    def SetData(self):
 
        try:
            wx.GetApp().SetInput(self.GetSteps(),self.GetValues())
            return True

        except:
            wx.MessageBox("Please insert correct input values", "Incorrect Values",wx.OK)
            return False      
            
    def GetValues(self):
        temp={ i.GetLabel().split()[0]:i.GetValue() for i in self.boxes}
        temp["output_folder"]=self.output_folder
        return temp

    def GetSteps(self):
        return [[float(self.list.GetItem(itemId=row, col=i).GetText()) for i in range( self.list.ColumnCount)] for row in range(self.count)]
        
    def WriteXML(self):
        dlg=wx.FileDialog(self, "Save", self.dirname, "", "*.xml*", wx.SAVE)
        if dlg.ShowModal() == wx.ID_OK:
            self.dirname=dlg.GetDirectory()

      
            Data = ET.Element("Data")
            General = ET.SubElement(Data,"General")
            for i in self.boxes:   
                info = ET.SubElement(General,"Value")
                info.set(i.GetLabel().split()[0],str(i.GetValue()))
                
            
            for i in self.GetSteps() :
                  Step=ET.SubElement(Data,"Steps")
                  Step.set("Step",str(i).replace(","," ").strip("[]"))
                
            tree = ET.ElementTree(Data)
            tree.write(dlg.GetDirectory()+"/"+dlg.GetFilename())
    
        dlg.Destroy()
    
    def LoadXML(self):    
        dlg=wx.FileDialog(self, "Load", self.dirname, "", "*.xml*", wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            doc=ET.parse(dlg.GetDirectory()+"/"+dlg.GetFilename())
            self.config.DeleteGroup("Path")
            self.config.Write("Path",dlg.GetDirectory())

            self.SetSteps([[float(j) for j in i.attrib["Step"].split()]for i in doc.findall("Steps")]) 
            
            self.SetBoxes([i.attrib.values()[0].strip("[]") for i in doc.findall(".//Value")])

        dlg.Destroy()  
        
        
        
