# -*- coding: utf-8 -*-
"""
Created on Wed Apr  9 22:18:08 2014

@author: lars
"""

from dolfin import *
from SubDomains import *
from PlasticModel import *
from ReturnMapping import *
import DataHandler as Handler
import utils as utils


        
def Calculations(Settings, Steps):
    set_log_level(20)
    debug=bool(Settings["Debug"]=="True")
    directory=Settings["output_folder"]

    update=Settings["Update"]
    """
    1. Assign General parameters  
    """
    """
    scaling:
        young           1.0e3
        Poisson         1
        SMYS            1  MPa auto scaled
        Work-hadrening  1.0e3 
    """
    
    outer_radi=Settings["Outer"]/2.0            
    inner_radi=outer_radi-Settings["Wall"]
    length=Settings["Length"]
    base=Settings["Base"]
    ratio= outer_radi/inner_radi
    Area= np.pi*(outer_radi**2 - inner_radi**2)
       
    
    nu =Settings["Poisson"]

    
    
    """
    Setting alpha
    """
    if Settings["End"]=="Closed":  
        alpha=1-2*nu
    elif Settings["End"]=="Open":
        alpha=-2*nu
    r_dep = (1+nu)*((1 -2*nu)*inner_radi + outer_radi**2/inner_radi) - alpha*nu*inner_radi    

    plastic_model=PlasticModel(1.0e3*Settings["Young"],Settings["Poisson"],Settings["SMYS"],1.0e3*Settings["Work-Hardening"])

    """
    2. Create Mesh
    """

            
    faces=int(Settings["Faces"])
    resolution=int(Settings["Resolution"])
    
    Inner_geo = Cylinder(Point(0,0,length- base),Point(0,0,base),inner_radi,faces)
    Outer_geo = Cylinder(Point(0,0,length -base),Point(0,0,base),outer_radi,faces)	
    
    mesh3d= Mesh(Outer_geo-Inner_geo, resolution)	
    
    if debug==True:
        print "Resolution  :", resolution
        print "Num. cells  :", mesh3d.num_cells()
        print "Num_vertices:", mesh3d.num_vertices()
        plot(mesh3d,interactive=True)


    """
    3.Define boundaries and subdomains
    """
    boundaries=FacetFunction("size_t",mesh3d)
    boundaries.set_all(0)
    tol=Settings["Wall"]/2
    subdomains=[Top(tol,height=length-base),Bottom(tol,base=base),Outer(tol ,outer_radi=outer_radi),Inner(tol ,inner_radi=inner_radi)]
                     
    """
    4.Marking boundaries with subdomains
    """              
    while any(boundaries.array()==0) : 
        subdomains[3].mark(boundaries ,4)     
        subdomains[2].mark(boundaries ,3)
        tol+=0.01        
        subdomains[2]=Outer(tol ,outer_radi=outer_radi)
        subdomains[3]=Inner(tol ,inner_radi=inner_radi)
        
    tol=0.01
    subdomains[0]=Top(tol,height=length-base)
    subdomains[1]=Bottom(tol,base=base)
    subdomains[0].mark(boundaries,1)
    subdomains[1].mark(boundaries,2) 
    
    """
    5.Specify the boundary integration in the weak formulation
    """
    ds = Measure("ds")[boundaries]

    if debug==True:
        plot(boundaries,interactive=True)
        print 2
################################################################################################################


    """
    6.Create the FunctionSpaces
    """
    quad= int(Settings["Quadrature"])
    V=VectorFunctionSpace(mesh3d,"Lagrange",quad)
    Vt=VectorFunctionSpace(mesh3d,"Lagrange",1,36)
    Vs=VectorFunctionSpace(mesh3d,"Lagrange",1,6)
    W= FunctionSpace(mesh3d, "Lagrange", 1)
    U= TensorFunctionSpace(mesh3d ,"Lagrange",1)
    Vx,Vy,Vz = V.split()
    
    
    
    """
    7.Store some auxillary values for easy access
    """

    num_points = Vs.dim()/Vs.element().value_dimension(0)
    num_nodes= V.dim()/V.element().value_dimension(0)
    print V.element().value_dimension(0)
    print Vs.element().value_dimension(0)
    """
    8.Intalize Functions
    """
    Cons_tangent=Function(Vt)
    Cons_tangent.vector().set_local(np.array(plastic_model.return_list()*num_points))
    
    Stress=Function(Vs)
    Stress.vector()[:]=0.0

    Strain_p=Function(Vs)
    Strain_p.vector()[:]=0.0
    
    Residual=Function(Vs)
    Residual.vector()[:]=0
    
    APS = Function(W)
    APS.vector()[:]=0.0

    Strain=Function(Vs)
    Strain.vector()[:]=0.0
  

    
    """
    9. Declear surfaces normals
    """
    normal=FacetNormal(mesh3d)
    
    
    """   
    10. The weak formulation
    """       
    u=TrialFunction(V)
    v=TestFunction(V)
    
    a=inner(utils.eps(v), dot(utils.tangent(Cons_tangent),utils.eps(u)) )*dx 

    L = inner(v,Constant((0.0,0.0,0.0)))*dx 

    
    """
    11.Pre-assemble of projection matrix, for faster projections
    """
    
    M = assemble(inner( TestFunction(Vs),utils.eps(TrialFunction(V)))*dx)
    M.compress()
    P= assemble(inner(TrialFunction(Vs), TestFunction(Vs))*dx)
    ones = Function(Vs)
    ones.vector()[:] = 1
    P_diag = P * ones.vector()
    P_diag.set_local(1.0/P_diag.array())

    """
    12. Iniate Return-mapping and DataHandler 
    """        
    RM=ReturnMapping(10000)    
    data=Handler.Data(num_points,directory)
    
    """
    Saving initial parameters
    """
    data.StoreGenereal(Settings,num_points,Area,num_nodes)
    data.StoreSteps(Steps)
    data.area_approx.append(sum(assemble(inner(TestFunction(V),Constant((1,0,0)))*ds(1))))
    data.StoreApproxArea()
    """
    13. Strain_increments
    """
    strain_increment=Settings["Increment"]
    
    """
    14. Problem function 
    """
    du = Function(V)
    u  = Function(V)
    dp=Function(V)
    p=Function(V)  

    """
    15.Expression to be used with Dirichlet conditions
    """
    
    Curvature= Expression("-x[2]*(CurvY*x[0]+CurvX*x[1])", CurvY=0.0,  CurvX=0.0) 
        
    Tension = Expression("x[2]*value",value=0.0 )
    
    """
    16. Initialize previous state variables
    """
    
    prv_curv_x=0.0
    prv_curv_y=0.0
    prv_tension=0.0
    prv_pressure=0.0
    for step in Steps:  
        """
            All float :
            Step number         = i[0]
            Curvature y         = i[1]
            Curvature x         = i[2]
            Internal Pressure   = i[3]
            Axial Tension       = i[4]
        """
        num_step=int(step[0])


        data.New_step()
        
        
        print (step[4]-prv_tension)
        if (step[4]-prv_tension)!=0:

            """
            Initilize based on loading or unloading
            """
            if (step[4]-prv_tension) >0:
                Tension.value=strain_increment   
                statement=utils.Loading
            elif (step[4]-prv_tension) < 0:
                Tension.value=-strain_increment 
                statement=utils.Unloading
                
                
                
            """
            Dirichlet conditions on boundary
            """
            BCS=[]
            BCS.append(DirichletBC(Vz,Tension,boundaries,2))
            BCS.append(DirichletBC(Vz,Tension,boundaries,1))
          
            force=sum(assemble(inner(TestFunction(V),dot(normal,utils.sigma_3(Stress)))*ds(1)))*1.5

            while statement(1.00e3*step[4],force)  :
                """
                new iteration 
                """
                data.New_iteration()
                """
                Assembling the problem
                """
                A=assemble(a)
                b=assemble(L)
 
                """
                Apply Dirichlet conditions
                """
                for i in BCS:
                    i.apply(A,b)
                """
                Solving problem
                """
                solve(A,du.vector(),b,"richardson","sor")
                
                u.vector()[:] += du.vector()
                grad_eps=M*u.vector() 
                Strain.vector()[:] = grad_eps* P_diag
                
                if update=="Increment": 
                    mesh3d.move(du)
                for point in range(num_points):
                    dofs_s=[point*6 + j for j in range(6)]
                    dofs_t=[point*36+ j for j in range(36)]

    
                    epsilon         =np.matrix(Strain.vector()[dofs_s]).T
                    epsilon_p       =np.matrix(Strain_p.vector()[dofs_s]).T
                    trial_sigma     =plastic_model.elastic_tangent*(epsilon-epsilon_p)
                    
                    tangent,data.e_p[point]=RM.automatic(plastic_model,trial_sigma,epsilon_p,data.e_p[point])
                    
                    data.MaxValues(trial_sigma,epsilon,epsilon_p)                

                    """
                    Updating functions
                    """
                    
                    Cons_tangent.vector()[dofs_t]=np.asarray(tangent).reshape(-1)
                    Stress.vector()[dofs_s]=np.asarray(trial_sigma).reshape(-1)
                    Strain_p.vector()[dofs_s]=np.asarray(epsilon_p).reshape(-1)
                 
                      
                data.AddValues()
   
                print max(data.e_p)
                
                f_x =sum(assemble(inner(TestFunction(V),dot(normal,utils.sigma_1(Stress)))*ds(1)))*1.5
                f_y= sum(assemble(inner(TestFunction(V),dot(normal,utils.sigma_2(Stress)))*ds(1)))*1.5
                f_z= sum(assemble(inner(TestFunction(V),dot(normal,utils.sigma_3(Stress)))*ds(1)))*1.5
                
                f_tot = (f_x**2+f_y**2+f_z**2)**0.5   
                
                print max(data.e_p),f_tot,f_z
                force=f_z
     

            prv_tension=1.0e3*step[4]
        
        
        
        
    

        """
        
        
        Setting curvature 
        """        

        total_curv_y=step[1]-prv_curv_y
        total_curv_x=step[2]-prv_curv_x
        prv_curv_x=step[2]
        prv_curv_y=step[1]

        if not (total_curv_x==0 and total_curv_y==0) :

            """
            Scaling and setting increment
            """
            max_iteration=(1.0e-3)*(total_curv_x**2 + total_curv_y**2)**0.5/strain_increment
            increment_curv_x = total_curv_x/max_iteration
            increment_curv_y = total_curv_y/max_iteration
            Curvature.CurvY=(1.0e-3)*increment_curv_y
            Curvature.CurvX=(1.0e-3)*increment_curv_x       

            """
            Dirichlet on top and bottom markes 1 and 2, in subdomain 0 and 1
            """     
            BCS = []

            BCS.append(DirichletBC(Vz,Curvature,boundaries,1))
            BCS.append(DirichletBC(Vz,Curvature,boundaries,2)) 
            

            iteration=0
 
            while iteration < max_iteration :
                """
                new iteration 
                """
                data.New_iteration()
                print iteration
                iteration+=1

                """
                Assembling the problem
                """
                A=assemble(a)
                b=assemble(L)
                
                """
                Apply Dirichlet conditions
                """
                for bcs in BCS:
                    bcs.apply(A,b)
                    
                    
                """
                Solving problem
                """
                solve(A,du.vector(),b,"richardson","sor")
                
                u.vector()[:] += du.vector()  
                
        
                grad_eps=M*u.vector() 
                Strain.vector()[:] = grad_eps* P_diag
                if update=="Increment": 
                    mesh3d.move(du)
                
                for point in range(num_points):

                    dofs_s=[point*6 + j for j in range(6)]
                    dofs_t=[point*36+ j for j in range(36)]

    
                    epsilon         =np.matrix(Strain.vector()[dofs_s]).T
                    epsilon_p       =np.matrix(Strain_p.vector()[dofs_s]).T
                    trial_sigma    =plastic_model.elastic_tangent*(epsilon-epsilon_p)
                    
                    tangent, data.e_p[point]=RM.automatic(plastic_model,trial_sigma,epsilon_p,data.e_p[point])
                    """
                    Store max values
                    """
                    data.MaxValues(trial_sigma,epsilon,epsilon_p)                
                   
  
                    """
                    Updating functions 
                    """
                    Cons_tangent.vector()[dofs_t]=np.asarray(tangent).reshape(-1)
                    Stress.vector()[dofs_s]=np.asarray(trial_sigma).reshape(-1)
                    Strain_p.vector()[dofs_s]=np.asarray(epsilon_p).reshape(-1)
            
                data.AddValues()

                print "Max APS :",max(data.e_p)
            
            


  


        if (step[3]-prv_pressure)!= 0.00:  

          
            """
            Setting Boundary conditions, 
            """
            

            dp.vector()[:]=assemble(inner(TestFunction(V),-normal)*ds(4)) 
            dp=utils.MakeUnitvector(dp)  
            
            
            BCS=[]    
            
            
            if   (step[3]-prv_pressure) > 0.00:
                Tension.value=alpha*strain_increment  
                BCS.append(DirichletBC(V,Constant(r_dep*strain_increment)*dp,boundaries,4))  
                statement=utils.Loading
            elif   (step[3]-prv_pressure) < 0.00: 
                Tension.value=alpha*(-strain_increment)  
                BCS.append(DirichletBC(V,Constant(r_dep*(-strain_increment))*dp,boundaries,4))
                statement=utils.Unloading
            
            
            BCS.append(DirichletBC(Vz,Tension,boundaries,1))
            BCS.append(DirichletBC(Vz,Tension,boundaries,2))
            BCS.append(DirichletBC(Vz,Tension,boundaries,4))
            
                
            p.vector()[:]=assemble(inner(TestFunction(V),dot(normal,utils.sigma_axis(Stress)))*ds(4))                   
            Area_4=sum(assemble(inner(TestFunction(V),Constant((1,0,0)))*ds(4)))
            pressure  = utils.SumVector(p)/Area_4            
            
        
            while statement(step[3],pressure)  :
                """
                new iteration 
                """
                data.New_iteration()
                """
                Assembling the problem
                """
                
                A=assemble(a)
                b=assemble(L)
                """
                Apply Dirichlet conditions
                """
                for bcs in BCS:
                    bcs.apply(A,b)
                """
                Solving problem
                """
                solve(A,du.vector(),b,"richardson","sor")
                
                u.vector()[:] += du.vector() 
                grad_eps=M*u.vector() 
                Strain.vector()[:] = grad_eps* P_diag
                
                if update=="Increment": 
                    mesh3d.move(du)
                for point in range(num_points):

                    dofs_s=[point*6 + j for j in range(6)]
                    dofs_t=[point*36+ j for j in range(36)]

           
                    epsilon     = np.matrix(Strain.vector()[dofs_s]).T
                    epsilon_p   = np.matrix(Strain_p.vector()[dofs_s]).T
                    trial_sigma = plastic_model.elastic_tangent*(epsilon-epsilon_p)
                    
                    tangent, data.e_p[point]=RM.automatic(plastic_model,trial_sigma,epsilon_p,data.e_p[point])
                    """
                    Store max values
                    """
                    data.MaxValues(trial_sigma,epsilon,epsilon_p)                
                    
                    """
                    Updating functions
                    """
                    Cons_tangent.vector()[dofs_t]=np.asarray(tangent).reshape(-1)
                    Stress.vector()[dofs_s]=np.asarray(trial_sigma).reshape(-1)
                    Strain_p.vector()[dofs_s]=np.asarray(epsilon_p).reshape(-1)

                p.vector()[:]=assemble(inner(TestFunction(V),dot(normal,utils.sigma_axis(Stress)))*ds(4))                   
                Area_4=sum(assemble(inner(TestFunction(V),Constant((1,0,0)))*ds(4)))
                pressure  = utils.SumVector(p)/Area_4         
            
                data.AddValues()
                
                print "Pressure :",pressure 
                print "Max APS :",max(data.e_p)
                print "Max Stress:",data.max_stress 
              
            prv_pressure=step[3]

        data.AddAPS()
        APS.vector()[:]=np.asarray(data.e_p).reshape(-1)

        """     
        Saving output
        """
        data.Plot(num_step)
        data.StoreValues(num_step)

        file = File(directory+"/Strain"+str(num_step)+".pvd")  
        file << project(sqrt((2.0/3.0)*dot(Strain,Strain)),W)
        file = File(directory+"/Stress"+str(num_step)+".pvd")
        file << project(sqrt((3.0/2.0)*dot(Stress,Stress)),W)
        file = File(directory+"/APS"+str(num_step)+".pvd")  
        file << APS
        file = File(directory+"/StrainTensor"+str(num_step)+".pvd")  
        file << project(utils.sigma(Strain),U)
        file = File(directory+"/StressTensor"+str(num_step)+".pvd")
        file << project(utils.sigma(Stress),U)
        file = File(directory+"/radial_Stress"+str(num_step)+".pvd")
        file << p

    data.StoreAPS()
    pass