import numpy as np
from PlasticModel import *
from scipy.optimize import *

def function(delta_lambda ,  plastic_model,trial_stress,e_p):
    D      =plastic_model.elastic_tangent
    P      =plastic_model.P
    H      =plastic_model.H
    sigma_y=plastic_model.yield_stress
    Q= 1.0*np.identity(6) + float(delta_lambda)*D*P
    Qinv = Q.I
    sigma= Qinv*trial_stress
    phi   =float(plastic_model.dg(sigma).T*plastic_model.dg(sigma))
    kappa =  e_p + float(delta_lambda)*(2.0/3.0*phi)**0.5
    return float( 0.5*phi - (sigma_y+kappa*H )**2/3.0 )

         
class ReturnMapping(): 
    def __init__(self, maxit=1000):
        self.maxit=maxit
        
    def automatic(self,plastic_model ,trial_stress,plastic_strain,e_p):
        delta_lambda=0.0
  
        De= plastic_model.elastic_tangent
        P=plastic_model.P
    
        if plastic_model.f(trial_stress,e_p)> 1.0e-12 :
            
            delta_lambda=float(brentq(function,0.0,1.0, args=(plastic_model,trial_stress,e_p, ),maxiter=self.maxit, full_output=False, disp=True ))
     
            Q= 1.0*np.identity(6) + float(delta_lambda)*De*P
            Qinv = Q.I
            
            sigma_current=Qinv*trial_stress
            

            
            trial_stress= sigma_current
            
            df_dsigma=plastic_model.df(trial_stress)
            dg_dsigma=plastic_model.dg(trial_stress)     

            hardening=plastic_model.hardening(trial_stress,e_p)

            Rn = (De*df_dsigma).T
            D = De - De*(dg_dsigma*Rn)/(Rn*dg_dsigma + hardening) 

            e_p=plastic_model.APS(trial_stress,delta_lambda,e_p)

            plastic_strain += delta_lambda*dg_dsigma

        else:
            D=De
           
            
        return D,e_p
        

