from dolfin import*
import numpy as np
def deviatoric(tensor_array):
    temp=tensor_array
    trace= tensor_array[0]+tensor_array[1]+tensor_array[2]
    temp[0]=tensor_array[0]-trace/3.0
    temp[1]=tensor_array[1]-trace/3.0
    temp[2]=tensor_array[2]-trace/3.0
    return temp

def equivalentStress(tensor_array):
    T=deviatoric(tensor_array)
    return 1.2247448713915892*(float(T[0])**2 + float(T[1])**2 + float(T[2])**2+2*float(T[3])**2+2*float(T[4])**2+2*float(T[5])**2)**0.5
    
def equivalentStrain(tensor_array):
    T=deviatoric(tensor_array)
    return 0.816496580927726*(float(T[0])**2 + float(T[1])**2 + float(T[2])**2+float(T[3])**2+float(T[4])**2+float(T[5])**2)**0.5
    
def eps(u):      
    	return as_vector([u[i].dx(i) for i in range(3)] + [u[i].dx(j) + u[j].dx(i) for i, j in [(0, 1), (0, 2), (1, 2)]])

def sigma(s):
    return as_matrix([[s[0], s[3], s[4]], [s[3], s[1], s[5]], [s[4], s[5], s[2]]])

def tangent(t):
  return as_matrix([[t[i*6 + j] for j in range(6)] for i in range(6)])
  
  
def sigma_axis(s):
    return as_matrix([[s[0], 0.0, 0.0], [0.0, s[1], 0.0], [0.0, 0.0, s[2]]])

def sigma_1(s):
    return as_matrix([[s[0], 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]])
def sigma_2(s):
    return as_matrix([[0.0, 0.0, 0.0], [0.0, s[1], 0.0], [0.0, 0.0, 0.0]])
def sigma_3(s):
    return as_matrix([[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, s[2]]])
    
def MakeUnitvector(Function):
    V = Function.function_space()
    num_points = V.dim()/V.element().value_dimension(0)
    for point in range(num_points):
        dofs_u =[point*3 + j for j in range(3)]
        v=Function.vector()[dofs_u]  
        if norm(v)!=0:  
            Function.vector()[dofs_u] /= norm(v)
            
    return Function
    
    
def SumVector(Function):
    V = Function.function_space()
    num_points = V.dim()/V.element().value_dimension(0)

    SumVector=0.0
    for point in range(num_points):
        dofs_u =[point*3 + j for j in range(3)]
        v=Function.vector()[dofs_u]  
        SumVector+=norm(v)
        
    return SumVector


    
def DeviatoricFunction(Function):
    V = Function.function_space()
    num_points = V.dim()/V.element().value_dimension(0)
    
    for point in range(num_points):
        dofs_u =[point*3 + j for j in range(3)]
        v=Function.vector()[dofs_u]  
        temp=(v[0]+v[1]+v[2])/3.0
        Function.vector()[dofs_u[0]]-=temp
        Function.vector()[dofs_u[1]]-=temp
        Function.vector()[dofs_u[2]]-=temp
        
    return Function
    
def Loading(step_input,variable):
    return (variable -step_input) < 0
    
    
def Unloading(step_input,variable):
    return  (step_input-variable) < 0
    